from __future__ import annotations

__codegen__ = True

from nshtrainer.profiler.pytorch import BaseProfilerConfig as BaseProfilerConfig
from nshtrainer.profiler.pytorch import PyTorchProfilerConfig as PyTorchProfilerConfig

__all__ = [
    "BaseProfilerConfig",
    "PyTorchProfilerConfig",
]
