from __future__ import annotations

__codegen__ = True

from nshtrainer.metrics import MetricConfig as MetricConfig

from . import _config as _config

__all__ = [
    "MetricConfig",
    "_config",
]
