from __future__ import annotations

__codegen__ = True

from nshtrainer.util.config.duration import DurationConfig as DurationConfig
from nshtrainer.util.config.duration import EpochsConfig as EpochsConfig
from nshtrainer.util.config.duration import StepsConfig as StepsConfig

__all__ = [
    "DurationConfig",
    "EpochsConfig",
    "StepsConfig",
]
