from __future__ import annotations

__codegen__ = True

from nshtrainer.trainer.strategy import StrategyConfig as StrategyConfig
from nshtrainer.trainer.strategy import StrategyConfigBase as StrategyConfigBase

__all__ = [
    "StrategyConfig",
    "StrategyConfigBase",
]
