from __future__ import annotations

__codegen__ = True

from nshtrainer.profiler.advanced import (
    AdvancedProfilerConfig as AdvancedProfilerConfig,
)
from nshtrainer.profiler.advanced import BaseProfilerConfig as BaseProfilerConfig

__all__ = [
    "AdvancedProfilerConfig",
    "BaseProfilerConfig",
]
