from __future__ import annotations

__codegen__ = True

from nshtrainer.profiler.simple import BaseProfilerConfig as BaseProfilerConfig
from nshtrainer.profiler.simple import SimpleProfilerConfig as SimpleProfilerConfig

__all__ = [
    "BaseProfilerConfig",
    "SimpleProfilerConfig",
]
